@@ -194,19 +194,24 @@ extern int c_debug;
     VAR_CATALOG = 395,             /* VAR_CATALOG  */
     VAR_CATALOG_MEMBER_PATTERN = 396, /* VAR_CATALOG_MEMBER_PATTERN  */
     VAR_CATALOG_PRODUCER_ZONE = 397, /* VAR_CATALOG_PRODUCER_ZONE  */
-    VAR_ZONE = 398,                /* VAR_ZONE  */
-    VAR_RRL_WHITELIST = 399,       /* VAR_RRL_WHITELIST  */
-    VAR_SERVERS = 400,             /* VAR_SERVERS  */
-    VAR_BINDTODEVICE = 401,        /* VAR_BINDTODEVICE  */
-    VAR_SETFIB = 402,              /* VAR_SETFIB  */
-    VAR_VERIFY = 403,              /* VAR_VERIFY  */
-    VAR_ENABLE = 404,              /* VAR_ENABLE  */
-    VAR_VERIFY_ZONE = 405,         /* VAR_VERIFY_ZONE  */
-    VAR_VERIFY_ZONES = 406,        /* VAR_VERIFY_ZONES  */
-    VAR_VERIFIER = 407,            /* VAR_VERIFIER  */
-    VAR_VERIFIER_COUNT = 408,      /* VAR_VERIFIER_COUNT  */
-    VAR_VERIFIER_FEED_ZONE = 409,  /* VAR_VERIFIER_FEED_ZONE  */
-    VAR_VERIFIER_TIMEOUT = 410     /* VAR_VERIFIER_TIMEOUT  */
+    VAR_XDP_INTERFACE = 398,       /* VAR_XDP_INTERFACE  */
+    VAR_XDP_PROGRAM_PATH = 399,    /* VAR_XDP_PROGRAM_PATH  */
+    VAR_XDP_PROGRAM_LOAD = 400,    /* VAR_XDP_PROGRAM_LOAD  */
+    VAR_XDP_BPFFS_PATH = 401,      /* VAR_XDP_BPFFS_PATH  */
+    VAR_XDP_FORCE_COPY = 402,      /* VAR_XDP_FORCE_COPY  */
+    VAR_ZONE = 403,                /* VAR_ZONE  */
+    VAR_RRL_WHITELIST = 404,       /* VAR_RRL_WHITELIST  */
+    VAR_SERVERS = 405,             /* VAR_SERVERS  */
+    VAR_BINDTODEVICE = 406,        /* VAR_BINDTODEVICE  */
+    VAR_SETFIB = 407,              /* VAR_SETFIB  */
+    VAR_VERIFY = 408,              /* VAR_VERIFY  */
+    VAR_ENABLE = 409,              /* VAR_ENABLE  */
+    VAR_VERIFY_ZONE = 410,         /* VAR_VERIFY_ZONE  */
+    VAR_VERIFY_ZONES = 411,        /* VAR_VERIFY_ZONES  */
+    VAR_VERIFIER = 412,            /* VAR_VERIFIER  */
+    VAR_VERIFIER_COUNT = 413,      /* VAR_VERIFIER_COUNT  */
+    VAR_VERIFIER_FEED_ZONE = 414,  /* VAR_VERIFIER_FEED_ZONE  */
+    VAR_VERIFIER_TIMEOUT = 415     /* VAR_VERIFIER_TIMEOUT  */
   };
   typedef enum yytokentype yytoken_kind_t;
 #endif
@@ -355,19 +360,24 @@ extern int c_debug;
 #define VAR_CATALOG 395
 #define VAR_CATALOG_MEMBER_PATTERN 396
 #define VAR_CATALOG_PRODUCER_ZONE 397
-#define VAR_ZONE 398
-#define VAR_RRL_WHITELIST 399
-#define VAR_SERVERS 400
-#define VAR_BINDTODEVICE 401
-#define VAR_SETFIB 402
-#define VAR_VERIFY 403
-#define VAR_ENABLE 404
-#define VAR_VERIFY_ZONE 405
-#define VAR_VERIFY_ZONES 406
-#define VAR_VERIFIER 407
-#define VAR_VERIFIER_COUNT 408
-#define VAR_VERIFIER_FEED_ZONE 409
-#define VAR_VERIFIER_TIMEOUT 410
+#define VAR_XDP_INTERFACE 398
+#define VAR_XDP_PROGRAM_PATH 399
+#define VAR_XDP_PROGRAM_LOAD 400
+#define VAR_XDP_BPFFS_PATH 401
+#define VAR_XDP_FORCE_COPY 402
+#define VAR_ZONE 403
+#define VAR_RRL_WHITELIST 404
+#define VAR_SERVERS 405
+#define VAR_BINDTODEVICE 406
+#define VAR_SETFIB 407
+#define VAR_VERIFY 408
+#define VAR_ENABLE 409
+#define VAR_VERIFY_ZONE 410
+#define VAR_VERIFY_ZONES 411
+#define VAR_VERIFIER 412
+#define VAR_VERIFIER_COUNT 413
+#define VAR_VERIFIER_FEED_ZONE 414
+#define VAR_VERIFIER_TIMEOUT 415
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -385,7 +395,7 @@ union YYSTYPE
   struct component *comp;
   int role;
 
-#line 389 "configparser.h"
+#line 399 "configparser.h"
 
 };
 typedef union YYSTYPE YYSTYPE;
