@@ -166,7 +166,7 @@ nsec3_has_soa(rr_type* rr)
 		rdata_atom_data(rr->rdatas[NSEC3_RDATA_BITMAP])[0] == 0 && /* first window = 0, */
 		/* [1]: bitmap length must be >= 1 */
 		/* [2]: bit[6] = SOA, thus mask first bitmap octet with 0x02 */
-		rdata_atom_data(rr->rdatas[NSEC3_RDATA_BITMAP])[2]&0x02) { /* SOA bit set */
+		(rdata_atom_data(rr->rdatas[NSEC3_RDATA_BITMAP])[2]&0x02)) { /* SOA bit set */
 		return 1;
 	}
 	return 0;
