@@ -203,6 +203,34 @@ query_create(region_type *region, uint16_t *compressed_dname_offsets,
 	return query;
 }
 
+query_type *
+query_create_with_buffer(region_type *region,
+	uint16_t *compressed_dname_offsets, size_t compressed_dname_size,
+	domain_type **compressed_dnames, struct buffer *buffer)
+{
+	query_type *query
+		= (query_type *) region_alloc_zero(region, sizeof(query_type));
+	/* create region with large block size, because the initial chunk
+	   saves many mallocs in the server */
+	query->region = region_create_custom(xalloc, free, 16384, 16384/8, 32, 0);
+	region_add_cleanup(region, query_cleanup, query);
+	query->compressed_dname_offsets = compressed_dname_offsets;
+	query->compressed_dnames = compressed_dnames;
+	query->packet = buffer;
+	query->compressed_dname_offsets_size = compressed_dname_size;
+	tsig_create_record(&query->tsig, region);
+	query->tsig_prepare_it = 1;
+	query->tsig_update_it = 1;
+	query->tsig_sign_it = 1;
+	return query;
+}
+
+void
+query_set_buffer_data(query_type *q, void *data, size_t data_capacity)
+{
+	buffer_create_from(q->packet, data, data_capacity);
+}
+
 void
 query_reset(query_type *q, size_t maxlen, int is_tcp)
 {
