@@ -212,8 +212,8 @@ rdata_unquoted_to_string(buffer_type *output, rdata_atom_type rdata,
 	for (i = 1; i <= length; ++i) {
 		char ch = (char) data[i];
 		if (isprint((unsigned char)ch)) {
-			if (ch == '"' || ch == '\\'
-			||  isspace((unsigned char)ch)) {
+			if (ch == '"' || ch == '\\' || ch == '(' || ch == ')'
+			  || ch == '\'' || isspace((unsigned char)ch)) {
 				buffer_printf(output, "\\");
 			}
 			buffer_printf(output, "%c", ch);
@@ -295,6 +295,15 @@ rdata_long_to_string(buffer_type *output, rdata_atom_type rdata,
 	return 1;
 }
 
+static int
+rdata_longlong_to_string(buffer_type *output, rdata_atom_type rdata,
+	rr_type* ATTR_UNUSED(rr))
+{
+	uint64_t data = read_uint64(rdata_atom_data(rdata));
+	buffer_printf(output, "%llu", (unsigned long long) data);
+	return 1;
+}
+
 static int
 rdata_a_to_string(buffer_type *output, rdata_atom_type rdata,
 	rr_type* ATTR_UNUSED(rr))
@@ -450,13 +459,13 @@ rdata_base32_to_string(buffer_type *output, rdata_atom_type rdata,
 
 static int
 rdata_base64_to_string(buffer_type *output, rdata_atom_type rdata,
-	rr_type* ATTR_UNUSED(rr))
+	rr_type* rr)
 {
 	int length;
 	size_t size = rdata_atom_size(rdata);
 	if(size == 0) {
 		/* single zero represents empty buffer */
-		buffer_write(output, "0", 1);
+		buffer_write(output, (rr->type == TYPE_DOA ? "-" : "0"), 1);
 		return 1;
 	}
 	buffer_reserve(output, size * 2 + 1);
@@ -607,30 +616,16 @@ rdata_services_to_string(buffer_type *output, rdata_atom_type rdata,
 static int
 rdata_ipsecgateway_to_string(buffer_type *output, rdata_atom_type rdata, rr_type* rr)
 {
-	int gateway_type = rdata_atom_data(rr->rdatas[1])[0];
-	switch(gateway_type) {
+	switch(rdata_atom_data(rr->rdatas[1])[0]) {
 	case IPSECKEY_NOGATEWAY:
 		buffer_printf(output, ".");
 		break;
 	case IPSECKEY_IP4:
-		rdata_a_to_string(output, rdata, rr);
-		break;
+		return rdata_a_to_string(output, rdata, rr);
 	case IPSECKEY_IP6:
-		rdata_aaaa_to_string(output, rdata, rr);
-		break;
+		return rdata_aaaa_to_string(output, rdata, rr);
 	case IPSECKEY_DNAME:
-		{
-			region_type* temp = region_create(xalloc, free);
-			const dname_type* d = dname_make(temp,
-				rdata_atom_data(rdata), 0);
-			if(!d) {
-				region_destroy(temp);
-				return 0;
-			}
-			buffer_printf(output, "%s", dname_to_string(d, NULL));
-			region_destroy(temp);
-		}
-		break;
+		return rdata_dns_name_to_string(output, rdata, rr);
 	default:
 		return 0;
 	}
@@ -974,6 +969,56 @@ rdata_hip_to_string(buffer_type *output, rdata_atom_type rdata,
 	return length != -1;
 }
 
+static int
+rdata_atma_to_string(buffer_type *output, rdata_atom_type rdata,
+	rr_type* ATTR_UNUSED(rr))
+{
+	uint16_t size = rdata_atom_size(rdata), i;
+
+	if(size < 2 || rdata_atom_data(rdata)[0] > 1)
+		return 0;
+	if(!rdata_atom_data(rdata)[0]) {
+		hex_to_string(output, rdata_atom_data(rdata) + 1, size - 1);
+		return 1;
+	}
+	for(i = 1; i < size; i++) {
+		if(!isdigit(rdata_atom_data(rdata)[i]))
+			return 0;
+	}
+	buffer_write_u8(output, '+');
+	buffer_write(output, rdata_atom_data(rdata) + 1, size - 1);
+	return 1;
+}
+
+static int
+rdata_amtrelay_d_type_to_string(buffer_type *output, rdata_atom_type rdata,
+	rr_type* ATTR_UNUSED(rr))
+{
+	uint8_t data = *rdata_atom_data(rdata);
+	buffer_printf(output , "%c %lu", (data & 0x80 ? '1' : '0'),
+			((unsigned long)data & 0x7f));
+	return 1;
+}
+
+static int
+rdata_amtrelay_relay_to_string(buffer_type *output, rdata_atom_type rdata,
+		rr_type* rr)
+{
+	switch(rdata_atom_data(rr->rdatas[1])[0] & 0x7f) {
+	case AMTRELAY_NOGATEWAY:
+		break;
+	case AMTRELAY_IP4:
+		return rdata_a_to_string(output, rdata, rr);
+	case AMTRELAY_IP6:
+		return rdata_aaaa_to_string(output, rdata, rr);
+	case AMTRELAY_DNAME:
+		return rdata_dns_name_to_string(output, rdata, rr);
+	default:
+		return 0;
+	}
+	return 1;
+}
+
 static int
 rdata_unknown_to_string(buffer_type *output, rdata_atom_type rdata,
 	rr_type* ATTR_UNUSED(rr))
@@ -992,6 +1037,7 @@ static rdata_to_string_type rdata_to_string_table[RDATA_ZF_UNKNOWN + 1] = {
 	rdata_byte_to_string,
 	rdata_short_to_string,
 	rdata_long_to_string,
+	rdata_longlong_to_string,
 	rdata_a_to_string,
 	rdata_aaaa_to_string,
 	rdata_rrtype_to_string,
@@ -1019,6 +1065,9 @@ static rdata_to_string_type rdata_to_string_table[RDATA_ZF_UNKNOWN + 1] = {
 	rdata_tag_to_string,
 	rdata_svcparam_to_string,
 	rdata_hip_to_string,
+	rdata_atma_to_string,
+	rdata_amtrelay_d_type_to_string,
+	rdata_amtrelay_relay_to_string,
 	rdata_unknown_to_string
 };
 
@@ -1077,6 +1126,9 @@ rdata_wireformat_to_rdata_atoms(region_type *region,
 		case RDATA_WF_LONG:
 			length = sizeof(uint32_t);
 			break;
+		case RDATA_WF_LONGLONG:
+			length = sizeof(uint64_t);
+			break;
 		case RDATA_WF_TEXTS:
 		case RDATA_WF_LONG_TEXT:
 			length = end - buffer_position(packet);
@@ -1154,6 +1206,26 @@ rdata_wireformat_to_rdata_atoms(region_type *region,
 				length += read_uint16(buffer_current(packet) + 2);
 			}
 			break;
+		case RDATA_WF_AMTRELAY_RELAY:
+			assert(i>1);
+			switch(rdata_atom_data(temp_rdatas[1])[0] & 0x7f) /* relay type */ {
+			default:
+			case AMTRELAY_NOGATEWAY:
+				length = 0;
+				break;
+			case AMTRELAY_IP4:
+				length = IP4ADDRLEN;
+				break;
+			case AMTRELAY_IP6:
+				length = IP6ADDRLEN;
+				break;
+			case AMTRELAY_DNAME:
+				is_domain = 1;
+				is_normalized = 1;
+				is_wirestore = 1;
+				break;
+			}
+			break;
 		}
 		if (is_domain) {
 			const dname_type *dname;
