@@ -24,14 +24,105 @@
 #include "xfrd.h"
 #include "xfrd-disk.h"
 #include "util.h"
-#ifdef HAVE_TLS_1_3
+#ifdef HAVE_SSL
 #include <openssl/ssl.h>
 #include <openssl/err.h>
+#include <openssl/x509.h>
+#include <openssl/evp.h>
 #endif
 
-#ifdef HAVE_TLS_1_3
+#ifdef HAVE_SSL
 void log_crypto_err(const char* str); /* in server.c */
 
+/* Extract certificate information for logging */
+void
+get_cert_info(SSL* ssl, region_type* region, char** cert_serial,
+              char** key_id, char** cert_algorithm, char** tls_version)
+{
+    X509* cert = NULL;
+    ASN1_INTEGER* serial = NULL;
+    EVP_PKEY* pkey = NULL;
+    unsigned char key_fingerprint[EVP_MAX_MD_SIZE];
+    unsigned int key_fingerprint_len = 0;
+    const EVP_MD* md = EVP_sha256();
+    const char* pkey_name = NULL;
+    const char* version_name = NULL;
+    int i;
+    char temp_buffer[1024]; /* Temporary buffer for serial number */
+
+    *cert_serial = NULL;
+    *key_id = NULL;
+    *cert_algorithm = NULL;
+    *tls_version = NULL;
+
+#ifdef HAVE_SSL_GET1_PEER_CERTIFICATE
+    cert = SSL_get1_peer_certificate(ssl);
+#else
+    cert = SSL_get_peer_certificate(ssl);
+#endif
+
+    if (!cert) {
+        return;
+    }
+
+    /* Get certificate serial number */
+    serial = X509_get_serialNumber(cert);
+    if (serial) {
+        BIGNUM* bn = ASN1_INTEGER_to_BN(serial, NULL);
+        if (bn) {
+            char* hex_serial = BN_bn2hex(bn);
+            if (hex_serial) {
+                snprintf(temp_buffer, sizeof(temp_buffer), "%s", hex_serial);
+                *cert_serial = region_strdup(region, temp_buffer);
+                OPENSSL_free(hex_serial);
+            }
+            BN_free(bn);
+        }
+    }
+
+    /* Get public key identifier (SHA-256 fingerprint) */
+    if (X509_pubkey_digest(cert, md, key_fingerprint, &key_fingerprint_len) == 1 && key_fingerprint_len >= 8) {
+        size_t id_len = 8; /* Use first 8 bytes as key identifier */
+        char key_id_buffer[17]; /* 8 bytes * 2 hex chars + null terminator */
+        for (i = 0; i < (int)id_len; i++) {
+            snprintf(key_id_buffer + (i * 2), sizeof(key_id_buffer) - (i * 2), "%02x", key_fingerprint[i]);
+        }
+        *key_id = region_strdup(region, key_id_buffer);
+    }
+
+    /* Get certificate algorithm using OpenSSL's native functions */
+    pkey = X509_get_pubkey(cert);
+    if (pkey) {
+#ifdef HAVE_EVP_PKEY_GET0_TYPE_NAME
+        pkey_name = EVP_PKEY_get0_type_name(pkey);
+#else
+        pkey_name = OBJ_nid2sn(EVP_PKEY_type(EVP_PKEY_id(pkey)));
+#endif
+        if (pkey_name) {
+            *cert_algorithm = region_strdup(region, pkey_name);
+        } else {
+            int pkey_type = EVP_PKEY_id(pkey);
+            char algo_buffer[32];
+            snprintf(algo_buffer, sizeof(algo_buffer), "Unknown(%d)", pkey_type);
+            *cert_algorithm = region_strdup(region, algo_buffer);
+        }
+        EVP_PKEY_free(pkey);
+    }
+
+    /* Get TLS version using OpenSSL's native function */
+    version_name = SSL_get_version(ssl);
+    if (version_name) {
+        *tls_version = region_strdup(region, version_name);
+    } else {
+        int version = SSL_version(ssl);
+        char version_buffer[16];
+        snprintf(version_buffer, sizeof(version_buffer), "Unknown(%d)", version);
+        *tls_version = region_strdup(region, version_buffer);
+    }
+
+    X509_free(cert);
+}
+
 static SSL_CTX*
 create_ssl_context()
 {
