@@ -43,6 +43,12 @@
 #include <systemd/sd-daemon.h>
 #endif
 
+#ifdef HAVE_SSL
+#include <openssl/ssl.h>
+#include <openssl/x509.h>
+#include <openssl/evp.h>
+#endif
+
 #define XFRD_UDP_TIMEOUT 10 /* seconds, before a udp request times out */
 #define XFRD_NO_IXFR_CACHE 172800 /* 48h before retrying ixfr's after notimpl */
 #define XFRD_MAX_ROUNDS 1 /* max number of rounds along the masters */
@@ -434,8 +440,17 @@ xfrd_shutdown()
 	if (xfrd->nsd->tls_ctx)
 		SSL_CTX_free(xfrd->nsd->tls_ctx);
 #  ifdef HAVE_TLS_1_3
-	if (xfrd->tcp_set->ssl_ctx)
+	if (xfrd->tcp_set->ssl_ctx) {
+		int i;
+		for(i=0; i<xfrd->tcp_set->tcp_max; i++) {
+			if(xfrd->tcp_set->tcp_state[i] &&
+				xfrd->tcp_set->tcp_state[i]->ssl) {
+				SSL_free(xfrd->tcp_set->tcp_state[i]->ssl);
+				xfrd->tcp_set->tcp_state[i]->ssl = NULL;
+			}
+		}
 		SSL_CTX_free(xfrd->tcp_set->ssl_ctx);
+	}
 #  endif
 #endif
 #ifdef USE_DNSTAP
@@ -2534,6 +2549,57 @@ xfrd_handle_received_xfr_packet(xfrd_zone_type* zone, buffer_type* packet)
 		buffer_printf(packet, " TSIG verified with key %s",
 			zone->master->key_options->name);
 	}
+
+	if(zone->master->tls_auth_options && zone->master->tls_auth_options->auth_domain_name) {
+		buffer_printf(packet, " TLS authenticated with domain %s",
+			zone->master->tls_auth_options->auth_domain_name);
+#ifdef HAVE_TLS_1_3
+		/* Get certificate information from the TLS connection */
+		if (zone->tcp_conn != -1) {
+			struct xfrd_tcp_pipeline* tp = NULL;
+			struct region* tmpregion = region_create(xalloc, free);
+			char *cert_serial=NULL, *key_id=NULL,
+				*cert_algorithm=NULL, *tls_version=NULL;
+			/* Find the pipeline for this zone */
+			for (int i = 0; i < xfrd->tcp_set->tcp_max; i++) {
+				struct xfrd_tcp_pipeline* test_tp = xfrd->tcp_set->tcp_state[i];
+				if (test_tp && test_tp->ssl && test_tp->handshake_done) {
+					/* Check if this pipeline is handling our zone */
+					struct xfrd_tcp_pipeline_id* zid;
+					RBTREE_FOR(zid, struct xfrd_tcp_pipeline_id*, test_tp->zone_per_id) {
+						if (zid->zone == zone) {
+							tp = test_tp;
+							break;
+						}
+					}
+					if (tp) break;
+				}
+			}
+			if(!tmpregion)
+				tp = NULL;
+			if(tp && tp->ssl) {
+				get_cert_info(tp->ssl, tmpregion, &cert_serial,
+					&key_id, &cert_algorithm, &tls_version);
+			} else {
+				tp = NULL;
+			}
+
+			if (tp && cert_serial && cert_serial[0] != '\0') {
+				buffer_printf(packet, " cert-serial:%s", cert_serial);
+			}
+			if (tp && key_id && key_id[0] != '\0') {
+				buffer_printf(packet, " key-id:%s", key_id);
+			}
+			if (tp && cert_algorithm && cert_algorithm[0] != '\0') {
+				buffer_printf(packet, " cert-algo:%s", cert_algorithm);
+			}
+			if (tp && tls_version && tls_version[0] != '\0') {
+				buffer_printf(packet, " tls-version:%s", tls_version);
+			}
+			region_destroy(tmpregion);
+		}
+#endif
+	}
 	buffer_flip(packet);
 	diff_write_commit(zone->apex_str, zone->latest_xfr->msg_old_serial,
 		zone->latest_xfr->msg_new_serial, zone->latest_xfr->msg_seq_nr, 1,
@@ -2630,7 +2696,7 @@ static void
 xfrd_handle_reload(int ATTR_UNUSED(fd), short event, void* ATTR_UNUSED(arg))
 {
 	/* reload timeout */
-	assert(event & EV_TIMEOUT);
+	assert((event & EV_TIMEOUT));
 	(void)event;
 	/* timeout wait period after this request is sent */
 	xfrd->reload_added = 0;
@@ -2952,7 +3018,7 @@ static void
 xfrd_handle_write_timer(int ATTR_UNUSED(fd), short event, void* ATTR_UNUSED(arg))
 {
 	/* timeout for write events */
-	assert(event & EV_TIMEOUT);
+	assert((event & EV_TIMEOUT));
 	(void)event;
 	if(xfrd->nsd->options->zonefiles_write == 0)
 		return;
@@ -2989,7 +3055,7 @@ static void xfrd_write_timer_set()
 static void xfrd_handle_child_timer(int ATTR_UNUSED(fd), short event,
 	void* ATTR_UNUSED(arg))
 {
-	assert(event & EV_TIMEOUT);
+	assert((event & EV_TIMEOUT));
 	(void)event;
 	/* only used to wakeup the process to reap children, note the
 	 * event is no longer registered */
