@@ -72,6 +72,8 @@ struct xfrd_state {
 	size_t zonestat_clear_num;
 	/* array of malloced entries with cumulative cleared stat values */
 	struct nsdst** zonestat_clear;
+	/* array of child_count size with cumulative cleared stat values */
+	struct nsdst* stat_clear;
 
 	/* timer for NSD reload */
 	struct timeval reload_timeout;
@@ -95,8 +97,9 @@ struct xfrd_state {
 	/* communication channel with server_main */
 	struct event ipc_handler;
 	int ipc_handler_flags;
-	struct xfrd_tcp *ipc_conn;
-	struct buffer* ipc_pass;
+	/* 2 * nsd->child_count communication channels from the serve childs */
+	struct event    *notify_events;
+	struct xfrd_tcp *notify_pipes;
 	/* sending ipc to server_main */
 	uint8_t need_to_send_shutdown;
 	uint8_t need_to_send_reload;
@@ -117,6 +120,12 @@ struct xfrd_state {
 	int notify_udp_num;
 	/* first and last notify_zone* entries waiting for a UDP socket */
 	struct notify_zone *notify_waiting_first, *notify_waiting_last;
+
+	/* tree of catalog consumer zones. Processing is disabled if > 1. */
+	rbtree_type *catalog_consumer_zones;
+
+	/* tree of updated catalog producer zones for which the content to serve */
+	rbtree_type *catalog_producer_zones;
 };
 
 /*
